@@ -1349,6 +1349,9 @@ gail_entry_get_keybinding (AtkAction *action,
                 label = GTK_ACCESSIBLE (target_object)->widget;
               } 
           }
+        else
+          label = NULL;
+
         g_object_unref (set);
 
         if (GTK_IS_LABEL (label))
@@ -1362,7 +1365,6 @@ gail_entry_get_keybinding (AtkAction *action,
         break;
       }
     default:
-      return_value = NULL;
       break;
     }
   return return_value; 
