@@ -322,6 +322,7 @@ find_label (GtkWidget *widget)
 {
   GList *labels;
   GtkWidget *label;
+  GtkWidget *temp_widget;
 
   labels = gtk_widget_list_mnemonic_labels (widget);
   label = NULL;
@@ -342,6 +343,22 @@ find_label (GtkWidget *widget)
       g_list_free (labels);
     }
 
+  /*
+   * Ignore a label within a button; bug #136602
+   */
+  if (label && GTK_IS_BUTTON (widget))
+    {
+      temp_widget = label;
+      while (temp_widget)
+        {
+          if (temp_widget == widget)
+            {
+              label = NULL;
+              break;
+            }
+          temp_widget = gtk_widget_get_parent (temp_widget);
+        }
+    } 
   return label;
 }
 
@@ -375,29 +392,53 @@ gail_widget_ref_relation_set (AtkObject *obj)
   if (!atk_relation_set_contains (relation_set, ATK_RELATION_LABELLED_BY))
     {
       label = find_label (widget);
-      if (label == NULL && GTK_IS_BUTTON (widget))
-        /*
-         * Handle the case where GnomneIconEntry is the mnemonic widget.
-         * The GtkButton which is a grandchild of the GnomeIconEntry
-         * should really be the mnemonic widget. See bug #133967.
-         */
+      if (label == NULL)
         {
-          GtkWidget *temp_widget;
-
-          temp_widget = gtk_widget_get_parent (widget);
-
-          if (GTK_IS_ALIGNMENT (temp_widget))
+          if (GTK_IS_BUTTON (widget))
+            /*
+             * Handle the case where GnomeIconEntry is the mnemonic widget.
+             * The GtkButton which is a grandchild of the GnomeIconEntry
+             * should really be the mnemonic widget. See bug #133967.
+             */
             {
-              temp_widget = gtk_widget_get_parent (temp_widget);
-              if (GTK_IS_BOX (temp_widget))
+              GtkWidget *temp_widget;
+
+              temp_widget = gtk_widget_get_parent (widget);
+
+              if (GTK_IS_ALIGNMENT (temp_widget))
                 {
-                  label = find_label (temp_widget);
+                  temp_widget = gtk_widget_get_parent (temp_widget);
+                  if (GTK_IS_BOX (temp_widget))
+                    {
+                      label = find_label (temp_widget);
                  
-                  if (!label)
-                    label = find_label (gtk_widget_get_parent (temp_widget));
+                      if (!label)
+                        label = find_label (gtk_widget_get_parent (temp_widget));
+                    }
+                }
+            }
+          else if (GTK_IS_COMBO (widget))
+            /*
+             * Handle the case when GnomeFileEntry is the mnemonic widget.
+             * The GnomeEntry which is a grandchild of the GnomeFileEntry
+             * should be the mnemonic widget. See bug #137584.
+             */
+            {
+              GtkWidget *temp_widget;
+
+              temp_widget = gtk_widget_get_parent (widget);
+
+              if (GTK_IS_HBOX (temp_widget))
+                {
+                  temp_widget = gtk_widget_get_parent (temp_widget);
+                  if (GTK_IS_BOX (temp_widget))
+                    {
+                      label = find_label (temp_widget);
+                    }
                 }
             }
         }
+
       if (label)
         {
 	  array [0] = gtk_widget_get_accessible (label);
